@@ -35,12 +35,16 @@
 #include <IOKit/IOMessage.h>
 #include <IOKit/IOKitKeys.h>
 
-// SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSICommandOperationCodes.h>
-#include <IOKit/scsi-commands/IOSCSIPrimaryCommandsDevice.h>
+// IOKit storage related headers
+#include <IOKit/storage/IOStorageDeviceCharacteristics.h>
 
+// SCSI Architecture Model Family includes
+#include <IOKit/scsi/SCSICommandOperationCodes.h>
+
+#include "IOSCSIPrimaryCommandsDevice.h"
 #include "SCSIPrimaryCommands.h"
 
+
 //
 //	Macros
 //
@@ -98,7 +102,9 @@ enum
 // Reserved fields
 #define fKeySwitchNotifier							fIOSCSIPrimaryCommandsDeviceReserved->fKeySwitchNotifier
 #define fANSIVersion								fIOSCSIPrimaryCommandsDeviceReserved->fANSIVersion
-
+#define fCMDQUE										fIOSCSIPrimaryCommandsDeviceReserved->fCMDQUE
+#define	fTaskID										fIOSCSIPrimaryCommandsDeviceReserved->fTaskID
+#define	fTaskIDLock									fIOSCSIPrimaryCommandsDeviceReserved->fTaskIDLock
 
 #if 0
 #pragma mark -
@@ -137,17 +143,15 @@ bool
 IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 {
 	
-	OSString *		string			= NULL;
+	bool			result			= false;
+	bool			supported		= false;
+	UInt64			maxByteCount	= 0;
 	OSIterator *	iterator		= NULL;
 	OSObject *		obj				= NULL;
-	bool			result			= false;
-	
 	OSDictionary *	dict			= NULL;
-	OSNumber *		timeOutValue 	= NULL;
-	
+	OSString *		string			= NULL;
 	OSNumber *		value			= NULL;
 	
-	
 	// Call our super class' start routine so that all inherited
 	// behavior is initialized.	
 	require ( super::start ( provider ), ErrorExit );
@@ -165,11 +169,14 @@ IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 	
 	fANSIVersion = kINQUIRY_ANSI_VERSION_NoClaimedConformance;
 	
+	fTaskIDLock = IOSimpleLockAlloc ( );
+	require_nonzero ( fTaskIDLock, FreeReservedMemory );
+	
 	fProtocolDriver = OSDynamicCast ( IOSCSIProtocolInterface, provider );
-	require_nonzero ( fProtocolDriver, FreeReservedMemory );
+	require_nonzero ( fProtocolDriver, FreeTaskIDLock );
 	
 	fDeviceCharacteristicsDictionary = OSDictionary::withCapacity ( 1 );
-	require_nonzero ( fDeviceCharacteristicsDictionary, FreeReservedMemory );
+	require_nonzero ( fDeviceCharacteristicsDictionary, FreeTaskIDLock );
 	
 	string = ( OSString * ) GetProtocolDriver ( )->getProperty ( kIOPropertySCSIVendorIdentification );	
 	check ( string );
@@ -267,7 +274,27 @@ IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 								OSBoolean,
 								service->getProperty ( kKeySwitchProperty ) );
 			
-			setProperty ( kAppleKeySwitchProperty, boolKey );
+			dict = GetProtocolCharacteristicsDictionary ( );
+			
+			OSString *	protocol = OSDynamicCast ( OSString, dict->getObject ( kIOPropertyPhysicalInterconnectTypeKey ) );
+			
+			if ( ( protocol == NULL ) ||
+				 ( protocol->isEqualTo ( kIOPropertyPhysicalInterconnectTypeUSB ) ) ||
+				 ( protocol->isEqualTo ( kIOPropertyPhysicalInterconnectTypeFireWire ) ) ||
+				 ( protocol->isEqualTo ( kIOPropertyPhysicalInterconnectTypeATAPI ) ) )
+			{			
+				
+				setProperty ( kAppleKeySwitchProperty, boolKey );
+				
+				// Add a notification for the Apple KeySwitch on the servers.
+				fKeySwitchNotifier = addNotification (
+						gIOMatchedNotification,
+						nameMatching ( kAppleKeySwitchProperty ),
+						( IOServiceNotificationHandler ) &IOSCSIPrimaryCommandsDevice::ServerKeyswitchCallback,
+						this,
+						0 );
+				
+			}
 			
 		}
 		
@@ -285,18 +312,18 @@ IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 	if ( dict != NULL )
 	{
 		
-		timeOutValue = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyReadTimeOutDurationKey ) );
+		value = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyReadTimeOutDurationKey ) );
 		
-		if ( timeOutValue != NULL )
+		if ( value != NULL )
 		{
-			fReadTimeoutDuration = timeOutValue->unsigned32BitValue ( );
+			fReadTimeoutDuration = value->unsigned32BitValue ( );
 		}
 		
-		timeOutValue = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyWriteTimeOutDurationKey ) );
+		value = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyWriteTimeOutDurationKey ) );
 		
-		if ( timeOutValue != NULL )
+		if ( value != NULL )
 		{
-			fWriteTimeoutDuration = timeOutValue->unsigned32BitValue ( );
+			fWriteTimeoutDuration = value->unsigned32BitValue ( );
 		}
 		
 	}
@@ -306,22 +333,42 @@ IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 	if ( dict != NULL )
 	{
 		
-		timeOutValue = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyReadTimeOutDurationKey ) );
+		value = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyReadTimeOutDurationKey ) );
 		
-		if ( timeOutValue != NULL )
+		if ( value != NULL )
 		{
-			fReadTimeoutDuration = timeOutValue->unsigned32BitValue ( );
+			fReadTimeoutDuration = value->unsigned32BitValue ( );
 		}
 		
-		timeOutValue = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyWriteTimeOutDurationKey ) );
+		value = OSDynamicCast ( OSNumber, dict->getObject ( kIOPropertyWriteTimeOutDurationKey ) );
 		
-		if ( timeOutValue != NULL )
+		if ( value != NULL )
 		{
-			fWriteTimeoutDuration = timeOutValue->unsigned32BitValue ( );
+			fWriteTimeoutDuration = value->unsigned32BitValue ( );
 		}
 		
 	}
 	
+	// See if the transport driver wants us to limit the read transfer byte count
+	supported = GetProtocolDriver ( )->IsProtocolServiceSupported (
+						kSCSIProtocolFeature_MaximumReadTransferByteCount,
+						&maxByteCount );	
+	
+	if ( supported == true )
+	{
+		setProperty ( kIOMaximumByteCountReadKey, maxByteCount, 64 );
+	}
+	
+	// See if the transport driver wants us to limit the write transfer byte count
+	supported = GetProtocolDriver ( )->IsProtocolServiceSupported (
+						kSCSIProtocolFeature_MaximumWriteTransferByteCount,
+						&maxByteCount );	
+	
+	if ( supported == true )
+	{
+		setProperty ( kIOMaximumByteCountWriteKey, maxByteCount, 64 );
+	}
+	
 	fDeviceAccessEnabled = true;
 	StartDeviceSupport ( );
 	
@@ -332,14 +379,6 @@ IOSCSIPrimaryCommandsDevice::start ( IOService * provider )
 		
 	}
 	
-	// Add a notification for the Apple KeySwitch on the servers.
-	fKeySwitchNotifier = addNotification (
-			gIOMatchedNotification,
-			nameMatching ( kAppleKeySwitchProperty ),
-			( IOServiceNotificationHandler ) &IOSCSIPrimaryCommandsDevice::ServerKeyswitchCallback,
-			this,
-			0 );
-	
 	result = true;
 	
 	return result;
@@ -354,11 +393,19 @@ CloseProvider:
 FreeDeviceDictionary:
 	
 	
-	require_nonzero ( fDeviceCharacteristicsDictionary, FreeReservedMemory );
+	require_nonzero ( fDeviceCharacteristicsDictionary, FreeTaskIDLock );
 	fDeviceCharacteristicsDictionary->release ( );
 	fDeviceCharacteristicsDictionary = NULL;
 	
 	
+FreeTaskIDLock:
+	
+	
+	require_nonzero ( fTaskIDLock, FreeReservedMemory );
+	IOSimpleLockFree ( fTaskIDLock );
+	fTaskIDLock = NULL;
+	
+	
 FreeReservedMemory:
 	
 	
@@ -366,7 +413,7 @@ FreeReservedMemory:
 	IODelete ( fIOSCSIPrimaryCommandsDeviceReserved, IOSCSIPrimaryCommandsDeviceExpansionData, 1 );
 	fIOSCSIPrimaryCommandsDeviceReserved = NULL;
 	
-	
+
 ErrorExit:
 	
 	
@@ -424,6 +471,14 @@ IOSCSIPrimaryCommandsDevice::free ( void )
 	if ( fIOSCSIPrimaryCommandsDeviceReserved != NULL )
 	{
 		
+		if ( fTaskIDLock != NULL )
+		{
+			
+			IOSimpleLockFree ( fTaskIDLock );
+			fTaskIDLock = NULL;
+			
+		}
+		
 		IODelete ( fIOSCSIPrimaryCommandsDeviceReserved, IOSCSIPrimaryCommandsDeviceExpansionData, 1 );
 		fIOSCSIPrimaryCommandsDeviceReserved = NULL;
 		
@@ -942,6 +997,27 @@ Exit:
 }
 
 
+//
+//  GetUniqueTagID - 	Returns a unique tagged task ID.			[PROTECTED]
+//
+
+SCSITaggedTaskIdentifier 
+IOSCSIPrimaryCommandsDevice::GetUniqueTagID ( void )
+{
+	
+	SCSITaggedTaskIdentifier	taskID = kSCSIUntaggedTaskIdentifier;
+	
+	IOSimpleLockLock ( fTaskIDLock );
+	
+	taskID = ++fTaskID;
+	
+	IOSimpleLockUnlock ( fTaskIDLock );
+	
+	return taskID;
+	
+}
+
+
 #if 0
 #pragma mark -
 #pragma mark  Supporting Object Accessor Methods
@@ -1225,6 +1301,45 @@ IOSCSIPrimaryCommandsDevice::GetTaskAttribute ( SCSITaskIdentifier request )
 }
 
 
+//
+//  SetTaggedTaskIdentifier - Sets the SCSITaggedTaskIdentifier.	[PROTECTED]
+//
+
+bool
+IOSCSIPrimaryCommandsDevice::SetTaggedTaskIdentifier (
+							SCSITaskIdentifier			request,
+							SCSITaggedTaskIdentifier	taggedTaskIdentifier )
+{
+	
+	SCSITask *	scsiRequest;
+	
+	scsiRequest = OSDynamicCast ( SCSITask, request );
+	check ( scsiRequest );
+	
+	return scsiRequest->SetTaggedTaskIdentifier ( taggedTaskIdentifier );
+	
+}
+
+
+//
+//  GetTaggedTaskIdentifier - Gets the SCSITaggedTaskIdentifier.	[PROTECTED]
+//
+
+SCSITaggedTaskIdentifier
+IOSCSIPrimaryCommandsDevice::GetTaggedTaskIdentifier (
+							SCSITaskIdentifier			request )
+{
+	
+	SCSITask *	scsiRequest;
+	
+	scsiRequest = OSDynamicCast ( SCSITask, request );
+	check ( scsiRequest );
+	
+	return scsiRequest->GetTaggedTaskIdentifier ( );
+	
+}
+
+
 //
 //  SetTaskState - Sets the SCSITaskState.							[PROTECTED]
 //
@@ -2014,6 +2129,32 @@ IOSCSIPrimaryCommandsDevice::SetANSIVersion ( UInt8 newVersion )
 }
 
 
+//
+//  GetCMDQUE - Gets the CMDQUE bit from the INQUIRY data.			[PROTECTED]
+//
+
+bool
+IOSCSIPrimaryCommandsDevice::GetCMDQUE ( void )
+{
+	
+	return fCMDQUE;
+	
+}
+
+
+//
+//  SetANSIVersion - Sets the CMDQUE bit from the INQUIRY data.	[PROTECTED]
+//
+
+void
+IOSCSIPrimaryCommandsDevice::SetCMDQUE ( bool value )
+{
+	
+	fCMDQUE = value;
+	
+}
+
+
 //
 //  GetModeSense - Called to get mode sense data from the device.	[PROTECTED]
 //
