@@ -29,7 +29,7 @@
 //
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/IOSCSIProtocolInterface.h>
+#include <IOKit/scsi/IOSCSIProtocolInterface.h>
 
 //
 //	Macros
