@@ -60,7 +60,7 @@ struct SCSICmd_INQUIRY_StandardData
 	UInt8		flags1;								// First byte of support flags
 	UInt8		flags2;								// Second byte of support flags (Byte 7)
 	char		VENDOR_IDENTIFICATION[kINQUIRY_VENDOR_IDENTIFICATION_Length];
-	char		PRODUCT_INDENTIFICATION[kINQUIRY_PRODUCT_IDENTIFICATION_Length];
+	char		PRODUCT_IDENTIFICATION[kINQUIRY_PRODUCT_IDENTIFICATION_Length];
 	char		PRODUCT_REVISION_LEVEL[kINQUIRY_PRODUCT_REVISION_LEVEL_Length];
 };
 typedef struct SCSICmd_INQUIRY_StandardData SCSICmd_INQUIRY_StandardData;
@@ -82,7 +82,7 @@ struct SCSICmd_INQUIRY_StandardDataAll
 	UInt8		flags1;								// First byte of support flags (Byte 6)
 	UInt8		flags2;								// Second byte of support flags (Byte 7)
 	char		VENDOR_IDENTIFICATION[kINQUIRY_VENDOR_IDENTIFICATION_Length];
-	char		PRODUCT_INDENTIFICATION[kINQUIRY_PRODUCT_IDENTIFICATION_Length];
+	char		PRODUCT_IDENTIFICATION[kINQUIRY_PRODUCT_IDENTIFICATION_Length];
 	char		PRODUCT_REVISION_LEVEL[kINQUIRY_PRODUCT_REVISION_LEVEL_Length];
 	
 	// Following is the optional data that may be returned by a device.
