@@ -56,8 +56,8 @@
 #include <IOKit/IOLib.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSICommandDefinitions.h>
-#include <IOKit/scsi-commands/SCSITask.h>
+#include <IOKit/scsi/SCSICommandDefinitions.h>
+#include <IOKit/scsi/SCSITask.h>
 #include "SCSITaskDefinition.h"
 
 //
