@@ -134,15 +134,27 @@ typedef enum SCSIServiceResponse
 
 
 /* Task Status Constants
- * The Task Status represents the completion status of the task. Only when a 
- * task completes with a service response of either TASK_COMPLETED or 
- * LINK_COMMAND_COMPLETE, will the status be considered valid.
+ * The Task Status represents the completion status of the task which provides the 
+ * SCSI Application Layer with additional information about how to procede in 
+ * handling a completed task.
+ *
+ * The SCSI Architecture Model specification only defines task status values for when
+ * a task completes with a service response of either TASK_COMPLETED or LINK_COMMAND_COMPLETE.
+ * Since additional information will aid in error recovery when a task fails to be completed
+ * by a device due to a service response of kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE,
+ * additional values have been defined that can be returned by the SCSI Protocol Layer to inform 
+ * the SCSI Application Layer of the cause of the delivery failure. 
  * 
  * The Task Status can only be modified by the SCSI Protocol Layer.  The SCSI
  * Application Layer can only read the status.
  */
 typedef enum SCSITaskStatus
 {
+	// These are the Task Status values as defined by the SCSI Architecture Model
+	// specification and are returned only when task completes with a service response 
+	// of either TASK_COMPLETED or LINK_COMMAND_COMPLETE.
+	// Since these are well documented in the SAM specifications, that document should be
+	// referenced for their meaning.
 	kSCSITaskStatus_GOOD						= 0x00,
 	kSCSITaskStatus_CHECK_CONDITION				= 0x02,
 	kSCSITaskStatus_CONDITION_MET				= 0x04,
@@ -153,11 +165,49 @@ typedef enum SCSITaskStatus
 	kSCSITaskStatus_TASK_SET_FULL				= 0x28,
 	kSCSITaskStatus_ACA_ACTIVE					= 0x30,
 	
+	// These are the Task Status values for use when a task fails to complete due to
+	// a service response of SERVICE_DELIVERY_OR_TARGET_FAILURE.
+	// These values are not defined in the SCSI specifications, but are here so that
+	// the SCSI Protocol Layer objects can provide more information, if available, to 
+	// the SCSI Application Layer as to the cause of the failure.
+	
+	// If a task is aborted by the SCSI Protocol Layer due to it exceeding the timeout
+	// value specified by the task, the task status shall be set to
+	// kSCSITaskStatus_TaskTimeoutOccurred.
+	kSCSITaskStatus_TaskTimeoutOccurred			= 0x01,
+	
+	// If a task is aborted by the SCSI Protocol Layer due to it exceeding a timeout
+	// value specified by the support for the protocol or a related specification, 
+	// the task status shall be set to kSCSITaskStatus_TaskTimeoutOccurred.
+	kSCSITaskStatus_ProtocolTimeoutOccurred		= 0x02,
+	
+	// If a task is unable to be delivered due to a failure of the device not
+	// accepting the task or the device acknowledging the attempt to send it
+	// the device the task status shall be set to kSCSITaskStatus_DeviceNotResponding.
+	// This will allow the SCSI Application driver to perform the necessary steps 
+	// to try to recover the device.  This shall only be reported after the 
+	// SCSI Protocol Layer driver has attempted all protocol specific attempts to recover the device.
+	kSCSITaskStatus_DeviceNotResponding			= 0x03,
+	
+	// If the task is unable to be delivered because the device has been detached,
+	// the task status shall be set to kSCSITaskStatus_DeviceNotPresent.  This will
+	// allow the SCSI Application Layer to halt the sending of tasks to the device and, 
+	// if supported, perform any device failover or system cleanup.
+	kSCSITaskStatus_DeviceNotPresent			= 0x04,
+
+	// If the task is unable to be delivered to the device due to a failure in the SCSI 
+	// Protocol Layer, such as a bus reset or communications error, but the device is
+	// is known to be functioning properly, the task status shall be set to 
+	// kSCSITaskStatus_DeliveryFailure.  This can also be reported if the task could not 
+	// be delivered due to a protocol error that has since been corrected.
+	kSCSITaskStatus_DeliveryFailure				= 0x05,
+            
 	// This status is not defined by the SCSI specifications,
 	// but is here to provide a status that can be returned in
 	// cases where there is not status available from the device
-	// server, for example, when the service response is neither
-	// TASK_COMPLETED nor LINK_COMMAND_COMPLETE.
+	// or protocol, for example, when the service response is neither
+	// TASK_COMPLETED nor LINK_COMMAND_COMPLETE or when the service response is
+	// SERVICE_DELIVERY_OR_TARGET_FAILURE and the reason for failure could not be determined.
 	kSCSITaskStatus_No_Status					= 0xFF
 } SCSITaskStatus;
 
@@ -200,7 +250,7 @@ enum
 #include <IOKit/IOMemoryDescriptor.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSICmds_REQUEST_SENSE_Defs.h>
+#include <IOKit/scsi/SCSICmds_REQUEST_SENSE_Defs.h>
 
 
 //
