@@ -253,45 +253,6 @@ ErrorExit:
 }
 
 
-//
-//	 doSyncReadWrite - Performs a synchronous read or write		   [PUBLIC]
-//
-
-IOReturn
-IOBlockStorageServices::doSyncReadWrite (
-				IOMemoryDescriptor * 	buffer,
-				UInt32 					block,
-				UInt32 					nblks )
-{
-	
-	IOReturn	status = kIOReturnNotAttached;
-	
-	// Return an error for incoming activity if we have been terminated
-	require ( isInactive ( ) == false, ErrorExit );
-	
-	// Make sure we don't go away while the command in being executed.
-	retain ( );
-	fProvider->retain ( );
-	
-	// Make sure our provider is in the correct power state to handle the I/O.
-	fProvider->CheckPowerState ( );	
-	
-	// Execute the command
-	status = fProvider->SyncReadWrite ( buffer, block, nblks, fProvider->ReportMediumBlockSize ( ) );
-	
-	// Release the retain for this command.	
-	fProvider->release ( );
-	release ( );
-	
-	
-ErrorExit:
-	
-	
-	return status;
-	
-}
-
-
 //
 //	 doEjectMedia - Ejects the media								   [PUBLIC]
 //
@@ -680,6 +641,76 @@ IOBlockStorageServices::reportWriteProtection ( bool * isWriteProtected )
 }
 
 
+//
+//	 getWriteCacheState - Gets write cache state					   [PUBLIC]
+//
+
+IOReturn
+IOBlockStorageServices::getWriteCacheState ( bool * enabled )
+{
+	
+	IOReturn	status = kIOReturnNotAttached;
+	
+	// Return an error for incoming activity if we have been terminated
+	require ( isInactive ( ) == false, ErrorExit );
+	
+	// Make sure we don't away while the command in being executed.
+	retain ( );
+	fProvider->retain ( );
+	
+	// Make sure our provider is in the correct power state to handle the I/O.	
+	fProvider->CheckPowerState ( );
+	
+	// Execute the command
+	status = fProvider->GetWriteCacheState ( enabled );
+	
+	// Release the retain for this command.	
+	fProvider->release ( );
+	release ( );
+	
+	
+ErrorExit:
+	
+	
+	return status;
+	
+}
+
+
+//
+//	 setWriteCacheState - Sets write cache state					   [PUBLIC]
+//
+
+IOReturn
+IOBlockStorageServices::setWriteCacheState ( bool enabled )
+{
+	
+	IOReturn	status = kIOReturnNotAttached;
+	
+	// Return an error for incoming activity if we have been terminated
+	require ( isInactive ( ) == false, ErrorExit );
+	
+	// Make sure we don't away while the command in being executed.
+	retain ( );
+	fProvider->retain ( );
+	
+	// Make sure our provider is in the correct power state to handle the I/O.	
+	fProvider->CheckPowerState ( );
+	
+	// Execute the command
+	status = fProvider->SetWriteCacheState ( enabled );
+	
+	// Release the retain for this command.	
+	fProvider->release ( );
+	release ( );
+	
+	
+ErrorExit:
+	
+	
+	return status;
+	
+}
 
 
 #if 0
