@@ -42,7 +42,7 @@
 #include <IOKit/storage/IOStorage.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/IOSCSIPrimaryCommandsDevice.h>
+#include <IOKit/scsi/IOSCSIPrimaryCommandsDevice.h>
 
 
 //
