@@ -29,7 +29,7 @@
 //
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSICommandOperationCodes.h>
+#include <IOKit/scsi/SCSICommandOperationCodes.h>
 #include "SCSIReducedBlockCommands.h"
 
 
