@@ -32,6 +32,9 @@
 #include "SCSITaskUserClient.h"
 #include "SCSITaskLib.h"
 
+// Libkern includes
+#include <libkern/OSAtomic.h>
+
 // IOKit includes
 #include <IOKit/IOCommandGate.h>
 #include <IOKit/IOWorkLoop.h>
@@ -40,11 +43,11 @@
 #include <IOKit/storage/IOBlockStorageDriver.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/IOSCSIPrimaryCommandsDevice.h>
-#include <IOKit/scsi-commands/IOSCSIMultimediaCommandsDevice.h>
-#include <IOKit/scsi-commands/SCSICmds_REQUEST_SENSE_Defs.h>
-#include <IOKit/scsi-commands/SCSICmds_INQUIRY_Definitions.h>
-#include <IOKit/scsi-commands/SCSICommandOperationCodes.h>
+#include <IOKit/scsi/IOSCSIPrimaryCommandsDevice.h>
+#include <IOKit/scsi/IOSCSIMultimediaCommandsDevice.h>
+#include <IOKit/scsi/SCSICmds_REQUEST_SENSE_Defs.h>
+#include <IOKit/scsi/SCSICmds_INQUIRY_Definitions.h>
+#include <IOKit/scsi/SCSICommandOperationCodes.h>
 
 
 //
@@ -256,6 +259,22 @@ SCSITaskUserClient::sMethods[kSCSITaskUserClientMethodCount] =
 		kIOUCStructIStructO,
 		sizeof ( AppleReadDVDStructureStruct ),
 		sizeof ( SCSITaskStatus )
+	},
+	{
+		// Method #20 SetCDSpeed
+		0,
+		( IOMethod ) &SCSITaskUserClient::SetCDSpeed,
+		kIOUCStructIStructO,
+		sizeof ( AppleSetCDSpeedStruct ),
+		sizeof ( SCSITaskStatus )
+	},
+	{
+		// Method #21 ReadFormatCapacities
+		0,
+		( IOMethod ) &SCSITaskUserClient::ReadFormatCapacities,
+		kIOUCStructIStructO,
+		sizeof ( AppleReadFormatCapacitiesStruct ),
+		sizeof ( SCSITaskStatus )
 	}
 };
 
@@ -458,46 +477,6 @@ SCSITaskUserClient::free ( void )
 }
 
 
-//
-//	 message - Handles termination messages.						[PUBLIC]
-//
-
-IOReturn
-SCSITaskUserClient::message ( UInt32 type, IOService * provider, void * arg )
-{
-	
-	IOReturn	status = kIOReturnSuccess;
-	
-	STATUS_LOG ( ( "%s::%s type = %ld, provider = %p\n",
-				 getName ( ), __FUNCTION__, type, provider ) );
-	
-	switch ( type )
-	{
-		
-		case kIOMessageServiceIsTerminated:
-		{	
-			
-			STATUS_LOG ( ( "kIOMessageServiceIsTerminated called\n" ) );
-			status = HandleTerminate ( provider );
-			
-		}
-		break;
-		
-		default:
-		{
-			
-			status = super::message ( type, provider, arg );
-			
-		}
-		break;
-		
-	}
-	
-	return status;
-	
-}
-
-
 //
 //	 IsExclusiveAccessAvailable - 	Determines if exclusive acces is available
 //									for this client.				[PUBLIC]
@@ -1147,6 +1126,10 @@ SCSITaskUserClient::Inquiry ( AppleInquiryStruct * 	inquiryData,
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1212,6 +1195,8 @@ SCSITaskUserClient::TestUnitReady ( vm_address_t 		senseDataBuffer,
 	task->release ( );
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	return status;
+	
 	
 TASK_SETUP_ERR:
 EXCLUSIVE_ACCESS_ERR:
@@ -1274,7 +1259,7 @@ SCSITaskUserClient::GetPerformance ( AppleGetPerformanceStruct * 	performanceDat
 									  0x00,
 									  ( performanceData->MAXIMUM_NUMBER_OF_DESCRIPTORS >> 8 )	& 0xFF,
 									    performanceData->MAXIMUM_NUMBER_OF_DESCRIPTORS			& 0xFF,
-									  0x00,
+									  performanceData->TYPE,
 									  0x00 );
 	
 	task->SetTimeoutDuration ( kThirtySecondTimeoutInMS );
@@ -1292,6 +1277,10 @@ SCSITaskUserClient::GetPerformance ( AppleGetPerformanceStruct * 	performanceDat
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1376,6 +1365,10 @@ SCSITaskUserClient::GetConfiguration ( AppleGetConfigurationStruct * 	configData
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1460,6 +1453,10 @@ SCSITaskUserClient::ModeSense10 ( AppleModeSense10Struct * 	modeSenseData,
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1546,6 +1543,10 @@ SCSITaskUserClient::SetWriteParametersModePage ( AppleWriteParametersModePageStr
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1724,6 +1725,10 @@ SCSITaskUserClient::ReadTableOfContents ( AppleReadTableOfContentsStruct * 	read
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1807,6 +1812,10 @@ SCSITaskUserClient::ReadDiscInformation ( AppleReadDiscInfoStruct * 	discInfoDat
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1890,6 +1899,10 @@ SCSITaskUserClient::ReadTrackInformation ( AppleReadTrackInfoStruct * 	trackInfo
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1975,6 +1988,10 @@ SCSITaskUserClient::ReadDVDStructure ( AppleReadDVDStructureStruct * 	dvdStructD
 	
 	*outStructSize = sizeof ( SCSITaskStatus );
 	
+	task->release ( );
+	
+	return status;
+	
 	
 BUFFER_PREPARATION_ERR:
 	
@@ -1994,6 +2011,169 @@ GENERAL_ERR:
 }
 
 
+//
+//	 SetCDSpeed - Called to set the new CD read/write speeds.
+//																	[PUBLIC]
+//
+
+IOReturn
+SCSITaskUserClient::SetCDSpeed ( AppleSetCDSpeedStruct *	setCDSpeedData,
+								 SCSITaskStatus * 			taskStatus,
+								 UInt32 					inStructSize,
+								 UInt32 * 					outStructSize )
+{
+	
+	IOReturn				status			= kIOReturnExclusiveAccess;
+	SCSITask * 				task			= NULL;
+	bool					state			= true;
+	UInt16					readSpeed		= 0;
+	UInt16					writeSpeed		= 0;
+	
+	
+	check ( setCDSpeedData );
+	check ( taskStatus );
+	check ( outStructSize );
+	
+	fOutstandingCommands++;
+	
+	require_action ( isInactive ( ) == false, GENERAL_ERR, status = kIOReturnNoDevice );
+	
+	state = fProtocolInterface->GetUserClientExclusivityState ( );
+	nrequire ( state, EXCLUSIVE_ACCESS_ERR );
+	
+	// Create and initialize a task
+	status = SetupTask ( &task );
+	require_success ( status, TASK_SETUP_ERR );
+	
+	readSpeed	= setCDSpeedData->LOGICAL_UNIT_READ_SPEED;
+	writeSpeed	= setCDSpeedData->LOGICAL_UNIT_WRITE_SPEED;
+	
+	task->SetCommandDescriptorBlock ( kSCSICmd_SET_CD_SPEED,
+									  0x00,
+									  ( readSpeed >>   8 )	& 0xFF,
+									  readSpeed				& 0xFF,
+									  ( writeSpeed >>  8 )	& 0xFF,
+									  writeSpeed			& 0xFF,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00 );
+	
+	task->SetTimeoutDuration ( kThirtySecondTimeoutInMS );
+	task->SetDataTransferDirection ( kSCSIDataTransfer_NoDataTransfer );
+	
+	status	= SendCommand ( task,
+							setCDSpeedData->senseDataBuffer,
+							taskStatus );
+	
+	*outStructSize = sizeof ( SCSITaskStatus );
+	
+	task->release ( );
+	
+	return status;
+	
+	
+TASK_SETUP_ERR:
+EXCLUSIVE_ACCESS_ERR:
+GENERAL_ERR:
+	
+	
+	fOutstandingCommands--;
+	
+	return status;
+	
+}
+
+
+//
+//	 ReadFormatCapacities - Issues a READ_FORMAT_CAPACITIES command to the
+//							 drive as defined in MMC-2.				[PUBLIC]
+//
+
+IOReturn
+SCSITaskUserClient::ReadFormatCapacities ( AppleReadFormatCapacitiesStruct *	readFormatCapacitiesData,
+										   SCSITaskStatus * 					taskStatus,
+										   UInt32 								inStructSize,
+										   UInt32 * 							outStructSize )
+{
+	
+	IOReturn				status			= kIOReturnExclusiveAccess;
+	IOReturn				status2			= kIOReturnExclusiveAccess;
+	bool					state			= true;
+	SCSITask * 				task			= NULL;
+	IOMemoryDescriptor *	buffer			= NULL;
+	
+	
+	check ( readFormatCapacitiesData );
+	check ( taskStatus );
+	check ( outStructSize );
+	
+	fOutstandingCommands++;
+	
+	require_action ( isInactive ( ) == false, GENERAL_ERR, status = kIOReturnNoDevice );
+	
+	state = fProtocolInterface->GetUserClientExclusivityState ( );
+	nrequire ( state, EXCLUSIVE_ACCESS_ERR );
+	
+	// Create and initialize a task
+	status = SetupTask ( &task );
+	require_success ( status, TASK_SETUP_ERR );
+	
+	// Prepare the buffers
+	status = PrepareBuffers ( &buffer, readFormatCapacitiesData->buffer, readFormatCapacitiesData->bufferSize, kIODirectionIn );
+	require_success ( status, BUFFER_PREPARATION_ERR );
+	
+	task->SetCommandDescriptorBlock ( kSCSICmd_READ_FORMAT_CAPACITIES,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00,
+									  0x00,
+									  ( readFormatCapacitiesData->bufferSize >> 8 ) & 0xFF,
+									    readFormatCapacitiesData->bufferSize        & 0xFF,
+									  0 );
+	
+	task->SetTimeoutDuration ( kThirtySecondTimeoutInMS );
+	task->SetDataTransferDirection ( kSCSIDataTransfer_FromTargetToInitiator );
+	task->SetDataBuffer ( buffer );
+	task->SetRequestedDataTransferCount ( readFormatCapacitiesData->bufferSize );
+	
+	status	= SendCommand ( task, readFormatCapacitiesData->senseDataBuffer, taskStatus );
+	
+	status2	= CompleteBuffers ( buffer );
+	
+	if ( ( status == kIOReturnSuccess ) && ( status2 != kIOReturnSuccess ) )
+	{
+		status = status2;
+	}
+	
+	*outStructSize = sizeof ( SCSITaskStatus );
+	
+	task->release ( );
+	
+	return status;
+	
+	
+BUFFER_PREPARATION_ERR:
+	
+	
+	task->release ( );
+	
+	
+TASK_SETUP_ERR:
+EXCLUSIVE_ACCESS_ERR:
+GENERAL_ERR:
+	
+	fOutstandingCommands--;
+	
+	return status;
+	
+}
+
+
 #if 0
 #pragma mark -
 #pragma mark Protected Methods
@@ -2020,7 +2200,7 @@ SCSITaskUserClient::getTargetAndMethodForIndex (
 	
 	fOutstandingCommands++;
 	
-	require ( isInactive ( ) == false, GENERAL_ERR );
+	require ( isInactive ( ) == false, DECREMENT_COUNTER );
 	
 	fProtocolInterface->retain ( );
 	
@@ -2035,11 +2215,15 @@ SCSITaskUserClient::getTargetAndMethodForIndex (
 	method 	= &sMethods[index];
 	
 	
-GENERAL_ERR:
+DECREMENT_COUNTER:
 	
 	
 	fOutstandingCommands--;
 	
+	
+GENERAL_ERR:
+	
+	
 	return method;
 	
 }
@@ -2066,7 +2250,7 @@ SCSITaskUserClient::getAsyncTargetAndMethodForIndex (
 	
 	fOutstandingCommands++;
 	
-	require ( isInactive ( ) == false, GENERAL_ERR );
+	require ( isInactive ( ) == false, DECREMENT_COUNTER );
 	
 	fProtocolInterface->retain ( );
 	
@@ -2081,11 +2265,15 @@ SCSITaskUserClient::getAsyncTargetAndMethodForIndex (
 	method 	= &sAsyncMethods[index];
 	
 	
-GENERAL_ERR:
+DECREMENT_COUNTER:
 	
 	
 	fOutstandingCommands--;
 	
+	
+GENERAL_ERR:
+	
+	
 	return method;
 	
 }
@@ -2366,6 +2554,23 @@ SCSITaskUserClient::GatedWaitForTask ( SCSITask * request )
 }
 
 
+//
+//	 didTerminate - Checks to see if termination should be deferred.
+//																	[PROTECTED]
+//
+
+bool
+SCSITaskUserClient::didTerminate ( IOService * 		provider,
+								   IOOptionBits		options,
+								   bool *			defer )
+{
+	
+	HandleTerminate ( provider );	
+	return true;
+	
+}
+
+
 //
 //	 HandleTerminate -	Handles terminating our object and any resources it
 //						allocated.									[PROTECTED]
@@ -2375,18 +2580,10 @@ IOReturn
 SCSITaskUserClient::HandleTerminate ( IOService * provider )
 {
 	
-	IOReturn		status 		= kIOReturnSuccess;
-	IOWorkLoop *	workLoop	= NULL;
+	IOReturn	status = kIOReturnSuccess;
 	
 	check ( provider );
 	
-	// Wait for any outstanding I/O to finish. No new I/O can occur since we
-	// are marked as terminated.
-	while ( fOutstandingCommands != 0 )
-	{
-		IOSleep ( 10 );
-	}
-	
 	// Check if we have our provider open.
 	if ( provider->isOpen ( this ) )
 	{
@@ -2396,12 +2593,7 @@ SCSITaskUserClient::HandleTerminate ( IOService * provider )
 		provider->close ( this, kIOSCSITaskUserClientAccessMask );
 		
 	}
-	
-	// Get the workloop before calling detach(), otherwise it won't work
-	// (it traverses the IORegistry and detach() removes us from the IORegistry).
-	workLoop = getWorkLoop ( );
-	check ( workLoop );
-	
+		
 	// Decouple us from the IORegistry.
 	detach ( provider );
 	fProvider = NULL;
@@ -2482,7 +2674,11 @@ SCSITaskUserClient::SendCommand ( SCSITask * 		request,
 							  ( void * ) request );
 	
 	*taskStatus = request->GetTaskStatus ( );
-	status = kIOReturnSuccess;
+	
+	if ( request->GetServiceResponse ( ) == kSCSIServiceResponse_TASK_COMPLETE )
+		status = kIOReturnSuccess;
+	else
+		status = kIOReturnIOError;
 	
 	
 ErrorExit:
@@ -2544,6 +2740,7 @@ SCSITaskUserClient::TaskCallback ( SCSITask * task, SCSITaskRefCon * refCon )
 	if ( refCon->commandType == kCommandTypeNonExclusive )
 	{
 		
+		fOutstandingCommands--;
 		fCommandGate->commandWakeup ( &refCon->commandType );
 		
 	}
